"""A creation workflow that adds an existing Layer 2 Circuit to the database."""

from typing import Self

from orchestrator import step
from orchestrator.forms import FormPage
from orchestrator.types import FormGenerator, State
from pydantic import BaseModel, ConfigDict, model_validator
from pydantic_forms.types import UUIDstr

from gso.products import ProductName
from gso.products.product_blocks.layer_2_circuit import Layer2CircuitType
from gso.products.product_types.layer_2_circuit import (
    ImportedLayer2Circuit,
    ImportedLayer2CircuitInactive,
    Layer2CircuitServiceType,
)
from gso.services.partners import get_partner_by_name
from gso.services.subscriptions import get_product_id_by_name
from gso.utils.types.interfaces import BandwidthString
from gso.utils.types.virtual_identifiers import VC_ID, VLAN_ID


def initial_input_form_generator() -> FormGenerator:
    """Generate a form that can be pre-filled using an :term:`API` endpoint."""

    class ServiceBindingPortInput(BaseModel):
        edge_port: UUIDstr
        geant_sid: str
        is_tagged: bool
        vlan_id: VLAN_ID
        custom_firewall_filters: bool

    class ImportLayer2CircuitForm(FormPage):
        model_config = ConfigDict(title="Import Layer 2 Circuit")

        service_type: Layer2CircuitServiceType
        partner: str
        layer_2_circuit_side_a: ServiceBindingPortInput
        layer_2_circuit_side_b: ServiceBindingPortInput
        virtual_circuit_id: VC_ID
        layer_2_circuit_type: Layer2CircuitType
        vlan_range_lower_bound: VLAN_ID | None = None
        vlan_range_upper_bound: VLAN_ID | None = None
        policer_enabled: bool = False
        bandwidth: BandwidthString | None = None

        @model_validator(mode="after")
        def partner_id_matches_edge_port_owner(self) -> Self:
            """Validate that the entered partner owns both Edge Ports."""
            partner_id = get_partner_by_name(self.partner)["partner_id"]
            if partner_id != self.layer_2_circuit_side_a.edge_port:
                msg = f"Selected Edge Port on side A is not owned by partner {self.partner}."
                raise ValueError(msg)
            if partner_id != self.layer_2_circuit_side_b.edge_port:
                msg = f"Selected Edge Port on side B is not owned by partner {self.partner}."
                raise ValueError(msg)
            return self

        @model_validator(mode="after")
        def tagged_layer_2_circuit_has_vlan_bounds(self) -> Self:
            """If a Layer 2 Circuit is tagged, it must have a :term:`VLAN` range set."""
            if self.layer_2_circuit_type == Layer2CircuitType.TAGGED and (
                self.vlan_range_lower_bound is None or self.vlan_range_upper_bound is None
            ):
                msg = (
                    f"A tagged Layer 2 Circuit must have a VLAN range set. Received lower: "
                    f"{self.vlan_range_lower_bound}, upper: {self.vlan_range_upper_bound}."
                )
                raise ValueError(msg)
            return self

    user_input = yield ImportLayer2CircuitForm
    return user_input.model_dump()


@step("Create subscription")
def create_subscription(partner: str, service_type: Layer2CircuitServiceType) -> State:
    """Create a new subscription object."""
    partner_id = get_partner_by_name(partner)["partner_id"]
    product_id = get_product_id_by_name(ProductName(service_type))
    subscription = ImportedLayer2Circuit.from_product_id(product_id, partner_id)

    return {"subscription": subscription, "subscription_id": subscription.subscription_id}


@step("Initialize subscription")
def initialize_subscription(
    subscription: ImportedLayer2CircuitInactive,
    vlan_range_lower_bound: VLAN_ID,
    vlan_range_upper_bound: VLAN_ID,
) -> State:
    """Initialize the subscription object."""
    subscription.layer_2_circuit.vlan_range_lower_bound = vlan_range_lower_bound
    subscription.layer_2_circuit.vlan_range_upper_bound = vlan_range_upper_bound
    return {"subscription": subscription}
